/*
 * varray_dml3.sql
 * Rozdzia 6, Oracle10g. Programowanie w jzyku PL/SQL
 * Ron Hardman, Mike McLaughlin i Scott Urman
 *
 * Ten skrypt demonstruje aktualizowanie elementw tablic varray
 */

SET ECHO ON
SET SERVEROUTPUT ON SIZE 1000000

-- Porzdkowanie rodowiska
BEGIN

  FOR i IN (SELECT   null
            FROM     addresses
            WHERE    address_id = 11) LOOP
    EXECUTE IMMEDIATE 'DELETE FROM addresses WHERE address_id = 11';
  END LOOP;

  FOR i IN (SELECT   null
            FROM     individuals
            WHERE    individual_id = 11) LOOP
    EXECUTE IMMEDIATE 'DELETE FROM individuals WHERE individual_id = 11';
  END LOOP;

  COMMIT;

END;
/

-- Wstawianie danych do tabeli individuals z powodu ograniczenia zwizanego z kluczem nadrzdnym
INSERT
INTO     individuals
VALUES
('11'
,'Jan'
,''
,'Kowalski');

-- Wstawianie danych do tabeli addresses przy uyciu tablicy varray
INSERT
INTO     addresses
VALUES
(11
,11
  ('Biuro senatora Kowalskiego'
  ,'Zachodnia 1'
  ,'Apartament 200')
,'Warszawa'
,'MAZ'
,'85-701'
,'PL');

-- Blok anonimowy demonstrujcy aktualizowanie elementw tablicy varray
DECLARE

  -- Definicja typu rekordowego reprezentujcego wiersze tabeli addresses
  TYPE address_type IS RECORD
  (address_id          INTEGER
  ,individual_id       INTEGER
  ,street_address      ADDRESS_VARRAY
  ,city                VARCHAR2(20 CHAR)
  ,state               VARCHAR2(20 CHAR)
  ,postal_code         VARCHAR2(20 CHAR)
  ,country_code        VARCHAR2(10 CHAR));

  -- Definicja zmiennej typu rekordowego reprezentujcego tabel addresses
  address              ADDRESS_TYPE;

  -- Definicja kursora zwracajcego atrybut  %ROWTYPE
  CURSOR get_street_address
    (address_id_in      INTEGER) IS
    SELECT   *
    FROM     addresses
    WHERE    address_id = address_id_in;

BEGIN

  -- Otwarcie kursora
  OPEN  get_street_address(11);

  -- Pobranie danych do zmiennej typu rekordowego
  FETCH get_street_address
  INTO  address;

  -- Zamknicie kursora
  CLOSE get_street_address;

  -- Modyfikacja pierwszego elementu zmiennej typu VARRAY 
  address.street_address(1) :=
    'Biuro senatora Jana Kowalskiego';

  -- Aktualizacja wartoci kolumny odpowiadajce typowi VARRAY 
  UPDATE   addresses
  SET      street_address = address.street_address
  WHERE    address_id = 11;

END;
/


-- Typ do wywietlania sformatowanych wartoci tablicy varray
CREATE OR REPLACE TYPE varray_nested_table
IS TABLE OF VARCHAR2(30 CHAR);
/

-- Zapytanie o warto kolumny
SELECT   column_value
FROM     THE (SELECT   CAST(street_address AS varray_nested_table)
              FROM     addresses
              WHERE    address_id = 11);
